==> chromium-49.0.2604.0.tar.xz: chromium-49.0.2604.0/chrome/android/java/src/org/chromium/chrome/browser/infobar/TranslateLanguagePanel.java <==
// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package org.chromium.chrome.browser.infobar;

import android.content.Context;
import android.graphics.Color;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.FrameLayout;
import android.widget.Spinner;
import android.widget.TextView;

import org.chromium.chrome.R;

import java.util.ArrayList;

/**
 * Language panel shown in the translate infobar.
 */
public class TranslateLanguagePanel
        implements TranslateSubPanel, AdapterView.OnItemSelectedListener {

    private static final int LANGUAGE_TYPE_SOURCE = 0;
    private static final int LANGUAGE_TYPE_TARGET = 1;

    // UI elements.
    private Spinner mSourceSpinner;
    private Spinner mTargetSpinner;

    // Items that are not interacted with.
    // Provided by the caller, the new languages will be set here if the user
    // clicks "done".
    private final TranslateOptions mOptions;

    // This object will be used to keep the state for the time the
    // panel is opened it can be totally discarded in the end if the user
    // clicks "cancel".
    private final TranslateOptions mSessionOptions;

    private LanguageArrayAdapter mSourceAdapter;
    private LanguageArrayAdapter mTargetAdapter;

    private final SubPanelListener mListener;

    /**
     * Display language drop downs so they can be picked as source or
     * target for a translation.
     *
     * @param listener triggered when the panel is closed
     * @param options will be modified with the new languages selected.
     */
    public TranslateLanguagePanel(SubPanelListener listener, TranslateOptions options) {
        mListener = listener;
        mOptions = options;
        mSessionOptions = new TranslateOptions(mOptions);
    }

    @Override
    public void createContent(Context context, InfoBarLayout layout) {
        mSourceSpinner = null;
        mTargetSpinner = null;

        String changeLanguage = context.getString(R.string.translate_infobar_change_languages);
        layout.setMessage(changeLanguage);

        // Set up the spinners.
        InfoBarControlLayout controlLayout = layout.addControlLayout();
        View sourceSpinnerView = controlLayout.addSpinner(R.id.translate_infobar_source_spinner);
        View targetSpinnerView = controlLayout.addSpinner(R.id.translate_infobar_target_spinner);

        mSourceSpinner =
                (Spinner) sourceSpinnerView.findViewById(R.id.translate_infobar_source_spinner);
        mTargetSpinner =
                (Spinner) targetSpinnerView.findViewById(R.id.translate_infobar_target_spinner);
        setUpSpinners(context);

        // Set up the buttons.
        layout.setButtons(context.getString(R.string.translate_button_done),
                context.getString(R.string.cancel));
    }

    @Override
    public void onButtonClicked(boolean primary) {
        if (primary) {
            mOptions.setSourceLanguage(mSessionOptions.sourceLanguageCode());
            mOptions.setTargetLanguage(mSessionOptions.targetLanguageCode());
        }
        mListener.onPanelClosed(ActionType.NONE);
    }

    private void setUpSpinners(Context context) {
        mSourceAdapter = new LanguageArrayAdapter(context, R.layout.translate_spinner,
                LANGUAGE_TYPE_SOURCE);
        mTargetAdapter = new LanguageArrayAdapter(context, R.layout.translate_spinner,
                LANGUAGE_TYPE_TARGET);

        // Determine how wide each spinner needs to be to avoid truncating its children.
        mSourceAdapter.addAll(createSpinnerLanguages(""));
        mTargetAdapter.addAll(createSpinnerLanguages(""));
        mSourceAdapter.measureWidthRequiredForView();
        mTargetAdapter.measureWidthRequiredForView();

        // Set up the spinners.
        mSourceSpinner.setOnItemSelectedListener(this);
        mTargetSpinner.setOnItemSelectedListener(this);
        mSourceSpinner.setAdapter(mSourceAdapter);
        mTargetSpinner.setAdapter(mTargetAdapter);
        reloadSpinners();
    }

    private void reloadSpinners() {
        mSourceAdapter.clear();
        mTargetAdapter.clear();

        mSourceAdapter.addAll(createSpinnerLanguages(mSessionOptions.targetLanguageCode()));
        mTargetAdapter.addAll(createSpinnerLanguages(mSessionOptions.sourceLanguageCode()));

        int originalSourceSelection = mSourceSpinner.getSelectedItemPosition();
        int newSourceSelection = getSelectionPosition(LANGUAGE_TYPE_SOURCE);
        if (originalSourceSelection != newSourceSelection) {
            mSourceSpinner.setSelection(newSourceSelection);
        }

        int originalTargetSelection = mTargetSpinner.getSelectedItemPosition();
        int newTargetSelection = getSelectionPosition(LANGUAGE_TYPE_TARGET);
        if (originalTargetSelection != newTargetSelection) {
            mTargetSpinner.setSelection(newTargetSelection);
        }
    }

    private int getSelectionPosition(int languageType) {
        String position_code = languageType == LANGUAGE_TYPE_SOURCE
                ? mSessionOptions.sourceLanguageCode()
                : mSessionOptions.targetLanguageCode();

        // Since the source and target languages cannot appear in both spinners,
        // the index for the source language can be off by one if comes after the
        // target language alphabetically (and vice versa).
        String opposite_code = languageType == LANGUAGE_TYPE_SOURCE
                ? mSessionOptions.targetLanguageCode()
                : mSessionOptions.sourceLanguageCode();

        int position = -1;
        int opposite = -1;

        for (int i = 0; i < mSessionOptions.allLanguages().size(); ++i) {
            if (mSessionOptions.allLanguages().get(i).mLanguageCode.equals(position_code)) {
                position = i;
            }
            if (mSessionOptions.allLanguages().get(i).mLanguageCode.equals(opposite_code)) {
                opposite = i;
            }
            if (opposite > -1 && position > -1) break;
        }
        if (opposite < position) position -= 1;

        return position;
    }

    @Override
    public void onItemSelected(AdapterView<?> adapter, View view, int position, long id) {
        Spinner spinner = (Spinner) adapter;
        String newCode = ((SpinnerLanguageElement) spinner.getSelectedItem()).getLanguageCode();
        if (spinner == mSourceSpinner) {
            mSessionOptions.setSourceLanguage(newCode);
        } else {
            mSessionOptions.setTargetLanguage(newCode);
        }
        reloadSpinners();
    }

    @Override
    public void onNothingSelected(AdapterView<?> adapter) {
    }

    /**
     * Determines what languages will be shown in the Spinner.
     * @param avoidCode ISO code of the language to avoid displaying.
     * Use "" to display all languages.
     */
    private ArrayList<SpinnerLanguageElement> createSpinnerLanguages(String avoidCode) {
        ArrayList<SpinnerLanguageElement> result = new ArrayList<SpinnerLanguageElement>();
        for (TranslateOptions.TranslateLanguagePair language : mSessionOptions.allLanguages()) {
            if (!language.mLanguageCode.equals(avoidCode)) {
                result.add(new SpinnerLanguageElement(
                        language.mLanguageRepresentation, language.mLanguageCode));
            }
        }
        return result;
    }

    /**
     * The drop down view displayed to show the currently selected value.
     */
    private static class LanguageArrayAdapter extends ArrayAdapter<SpinnerLanguageElement> {
        private final SpannableString mTextTemplate;
        private int mMinimumWidth;

        public LanguageArrayAdapter(Context context, int textViewResourceId,
                int languageType) {
            super(context, textViewResourceId);

            // Get the string that we will display inside the Spinner, indicating whether the
            // spinner is used for the source or target language.
            String textTemplate = languageType == LANGUAGE_TYPE_SOURCE
                    ? context.getString(R.string.translate_options_source_hint)
                    : context.getString(R.string.translate_options_target_hint);
            mTextTemplate = new SpannableString(textTemplate);
            mTextTemplate.setSpan(
                    new ForegroundColorSpan(Color.GRAY), 0, textTemplate.length(), 0);
        }

        /** Measures how large the view needs to be to avoid truncating its children. */
        public void measureWidthRequiredForView() {
            mMinimumWidth = 0;

            final int spec = View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED);

            FrameLayout layout = new FrameLayout(getContext());
            TextView estimator = (TextView) LayoutInflater.from(getContext()).inflate(
                    R.layout.infobar_text, null);
            layout.addView(estimator);
            for (int i = 0; i < getCount(); ++i) {
                estimator.setText(getStringForLanguage(i));
                estimator.measure(spec, spec);
                mMinimumWidth = Math.max(mMinimumWidth, estimator.getMeasuredWidth());
            }
        }

        @Override
        public View getDropDownView(int position, View convertView, ViewGroup parent) {
            View result = super.getDropDownView(position, convertView, parent);
            if (result instanceof TextView) {
                ((TextView) result).setText(getItem(position).toString());
            }
            return result;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            TextView result;
            if (!(convertView instanceof TextView)) {
                result = new TextView(getContext());
                result.setTextSize(TypedValue.COMPLEX_UNIT_PX,
                        getContext().getResources().getDimension(R.dimen.infobar_text_size));
            } else {
                result = (TextView) convertView;
            }
            result.setEllipsize(TextUtils.TruncateAt.END);
            result.setMaxLines(1);
            result.setText(getStringForLanguage(position));
            result.setMinWidth(mMinimumWidth);
            return result;
        }

        private CharSequence getStringForLanguage(int position) {
            // The spinners prepend a string to show if they're for the source or target language.
            String language = getItem(position).toString();
            SpannableString lang = new SpannableString(language);
            lang.setSpan(new ForegroundColorSpan(Color.BLACK), 0, lang.length(), 0);
            return TextUtils.expandTemplate(mTextTemplate, lang);
        }
    }

    /**
     * The element that goes inside the spinner.
     */
    private static class SpinnerLanguageElement {
        private final String mLanguageName;
        private final String mLanguageCode;

        public SpinnerLanguageElement(String languageName, String languageCode) {
            mLanguageName = languageName;
            mLanguageCode = languageCode;
        }

        public String getLanguageCode() {
            return mLanguageCode;
        }

        /**
         * This is the text displayed in the spinner element so make sure no debug information
         * is added.
         */
        @Override
        public String toString() {
            return mLanguageName;
        }
    }
}

==> chromium-49.0.2619.0.tar.xz: chromium-49.0.2619.0/chrome/android/java/src/org/chromium/chrome/browser/infobar/TranslateLanguagePanel.java <==
// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package org.chromium.chrome.browser.infobar;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;

import org.chromium.chrome.R;

import java.util.ArrayList;

/**
 * Language panel shown in the translate infobar.
 */
public class TranslateLanguagePanel
        implements TranslateSubPanel, AdapterView.OnItemSelectedListener {

    private static final int LANGUAGE_TYPE_SOURCE = 0;
    private static final int LANGUAGE_TYPE_TARGET = 1;

    // UI elements.
    private Spinner mSourceSpinner;
    private Spinner mTargetSpinner;

    // Items that are not interacted with.
    // Provided by the caller, the new languages will be set here if the user
    // clicks "done".
    private final TranslateOptions mOptions;

    // This object will be used to keep the state for the time the
    // panel is opened it can be totally discarded in the end if the user
    // clicks "cancel".
    private final TranslateOptions mSessionOptions;

    private LanguageArrayAdapter mSourceAdapter;
    private LanguageArrayAdapter mTargetAdapter;

    private final SubPanelListener mListener;

    /**
     * Display language drop downs so they can be picked as source or
     * target for a translation.
     *
     * @param listener triggered when the panel is closed
     * @param options will be modified with the new languages selected.
     */
    public TranslateLanguagePanel(SubPanelListener listener, TranslateOptions options) {
        mListener = listener;
        mOptions = options;
        mSessionOptions = new TranslateOptions(mOptions);
    }

    @Override
    public void createContent(Context context, InfoBarLayout layout) {
        mSourceSpinner = null;
        mTargetSpinner = null;

        String changeLanguage = context.getString(R.string.translate_infobar_change_languages);
        layout.setMessage(changeLanguage);

        // Set up the spinners.
        InfoBarControlLayout controlLayout = layout.addControlLayout();
        View sourceSpinnerView = controlLayout.addSpinner(R.id.translate_infobar_source_spinner);
        View targetSpinnerView = controlLayout.addSpinner(R.id.translate_infobar_target_spinner);

        mSourceSpinner =
                (Spinner) sourceSpinnerView.findViewById(R.id.translate_infobar_source_spinner);
        mTargetSpinner =
                (Spinner) targetSpinnerView.findViewById(R.id.translate_infobar_target_spinner);
        setUpSpinners(context);

        // Set up the buttons.
        layout.setButtons(context.getString(R.string.translate_button_done),
                context.getString(R.string.cancel));
    }

    @Override
    public void onButtonClicked(boolean primary) {
        if (primary) {
            mOptions.setSourceLanguage(mSessionOptions.sourceLanguageCode());
            mOptions.setTargetLanguage(mSessionOptions.targetLanguageCode());
        }
        mListener.onPanelClosed(ActionType.NONE);
    }

    private void setUpSpinners(Context context) {
        mSourceAdapter = new LanguageArrayAdapter(context, R.layout.translate_spinner,
                LANGUAGE_TYPE_SOURCE);
        mTargetAdapter = new LanguageArrayAdapter(context, R.layout.translate_spinner,
                LANGUAGE_TYPE_TARGET);

        // Set up the spinners.
        mSourceSpinner.setOnItemSelectedListener(this);
        mTargetSpinner.setOnItemSelectedListener(this);
        mSourceSpinner.setAdapter(mSourceAdapter);
        mTargetSpinner.setAdapter(mTargetAdapter);
        reloadSpinners();
        measureSpinnerMinWidths();
    }

    private void reloadSpinners() {
        mSourceAdapter.clear();
        mTargetAdapter.clear();

        mSourceAdapter.addAll(createSpinnerLanguages(mSessionOptions.targetLanguageCode()));
        mTargetAdapter.addAll(createSpinnerLanguages(mSessionOptions.sourceLanguageCode()));

        int originalSourceSelection = mSourceSpinner.getSelectedItemPosition();
        int newSourceSelection = getSelectionPosition(LANGUAGE_TYPE_SOURCE);
        if (originalSourceSelection != newSourceSelection) {
            mSourceSpinner.setSelection(newSourceSelection);
        }

        int originalTargetSelection = mTargetSpinner.getSelectedItemPosition();
        int newTargetSelection = getSelectionPosition(LANGUAGE_TYPE_TARGET);
        if (originalTargetSelection != newTargetSelection) {
            mTargetSpinner.setSelection(newTargetSelection);
        }
    }

    private int getSelectionPosition(int languageType) {
        String position_code = languageType == LANGUAGE_TYPE_SOURCE
                ? mSessionOptions.sourceLanguageCode()
                : mSessionOptions.targetLanguageCode();

        // Since the source and target languages cannot appear in both spinners,
        // the index for the source language can be off by one if comes after the
        // target language alphabetically (and vice versa).
        String opposite_code = languageType == LANGUAGE_TYPE_SOURCE
                ? mSessionOptions.targetLanguageCode()
                : mSessionOptions.sourceLanguageCode();

        int position = -1;
        int opposite = -1;

        for (int i = 0; i < mSessionOptions.allLanguages().size(); ++i) {
            if (mSessionOptions.allLanguages().get(i).mLanguageCode.equals(position_code)) {
                position = i;
            }
            if (mSessionOptions.allLanguages().get(i).mLanguageCode.equals(opposite_code)) {
                opposite = i;
            }
            if (opposite > -1 && position > -1) break;
        }
        if (opposite < position) position -= 1;

        return position;
    }

    @Override
    public void onItemSelected(AdapterView<?> adapter, View view, int position, long id) {
        Spinner spinner = (Spinner) adapter;
        String newCode = ((SpinnerLanguageElement) spinner.getSelectedItem()).getLanguageCode();
        if (spinner == mSourceSpinner && !newCode.equals(mSessionOptions.sourceLanguageCode())) {
            mSessionOptions.setSourceLanguage(newCode);
            reloadSpinners();
        }
        if (spinner == mTargetSpinner && !newCode.equals(mSessionOptions.targetLanguageCode())) {
            mSessionOptions.setTargetLanguage(newCode);
            reloadSpinners();
        }
    }

    @Override
    public void onNothingSelected(AdapterView<?> adapter) {
    }

    /**
     * Determines what languages will be shown in the Spinner.
     * @param avoidCode ISO code of the language to avoid displaying.
     * Use "" to display all languages.
     */
    private ArrayList<SpinnerLanguageElement> createSpinnerLanguages(String avoidCode) {
        ArrayList<SpinnerLanguageElement> result = new ArrayList<SpinnerLanguageElement>();
        for (TranslateOptions.TranslateLanguagePair language : mSessionOptions.allLanguages()) {
            if (!language.mLanguageCode.equals(avoidCode)) {
                result.add(new SpinnerLanguageElement(
                        language.mLanguageRepresentation, language.mLanguageCode));
            }
        }
        return result;
    }

    /** Measures how wide the spinners need to be to avoid truncating text. */
    private void measureSpinnerMinWidths() {
        Context context = mSourceAdapter.getContext();
        TextView textView = (TextView) mSourceAdapter.getView(0, null, null);
        Paint textPaint = textView.getPaint();

        float longestLanguageWidth = 0;
        for (TranslateOptions.TranslateLanguagePair language : mSessionOptions.allLanguages()) {
            float width = textPaint.measureText(language.mLanguageRepresentation);
            longestLanguageWidth = Math.max(longestLanguageWidth, width);
        }

        float sourceTemplateWidth = textPaint.measureText(TextUtils.expandTemplate(
                context.getString(R.string.translate_options_source_hint), "").toString());
        float targetTemplateWidth = textPaint.measureText(TextUtils.expandTemplate(
                context.getString(R.string.translate_options_target_hint), "").toString());

        mSourceAdapter.mMinWidth = (int) Math.ceil(sourceTemplateWidth + longestLanguageWidth);
        mTargetAdapter.mMinWidth = (int) Math.ceil(targetTemplateWidth + longestLanguageWidth);
    }

    /**
     * The drop down view displayed to show the currently selected value.
     */
    private static class LanguageArrayAdapter extends ArrayAdapter<SpinnerLanguageElement> {
        private final SpannableString mTextTemplate;
        private int mMinWidth;

        public LanguageArrayAdapter(Context context, int textViewResourceId,
                int languageType) {
            super(context, textViewResourceId);

            // Get the string that we will display inside the Spinner, indicating whether the
            // spinner is used for the source or target language.
            String textTemplate = languageType == LANGUAGE_TYPE_SOURCE
                    ? context.getString(R.string.translate_options_source_hint)
                    : context.getString(R.string.translate_options_target_hint);
            mTextTemplate = new SpannableString(textTemplate);
            mTextTemplate.setSpan(
                    new ForegroundColorSpan(Color.GRAY), 0, textTemplate.length(), 0);
        }

        @Override
        public View getDropDownView(int position, View convertView, ViewGroup parent) {
            TextView view = (TextView) super.getDropDownView(position, convertView, parent);
            view.setText(getItem(position).toString());
            return view;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            TextView result;
            if (convertView instanceof TextView) {
                result = (TextView) convertView;
            } else {
                result = new TextView(getContext());
                result.setTextSize(TypedValue.COMPLEX_UNIT_PX,
                        getContext().getResources().getDimension(R.dimen.infobar_text_size));
                result.setEllipsize(TextUtils.TruncateAt.END);
                result.setMaxLines(1);
                result.setMinWidth(mMinWidth);
            }
            result.setText(getStringForLanguage(position));
            return result;
        }

        private CharSequence getStringForLanguage(int position) {
            // The spinners prepend a string to show if they're for the source or target language.
            String language = getItem(position).toString();
            SpannableString lang = new SpannableString(language);
            lang.setSpan(new ForegroundColorSpan(Color.BLACK), 0, lang.length(), 0);
            return TextUtils.expandTemplate(mTextTemplate, lang);
        }
    }

    /**
     * The element that goes inside the spinner.
     */
    private static class SpinnerLanguageElement {
        private final String mLanguageName;
        private final String mLanguageCode;

        public SpinnerLanguageElement(String languageName, String languageCode) {
            mLanguageName = languageName;
            mLanguageCode = languageCode;
        }

        public String getLanguageCode() {
            return mLanguageCode;
        }

        /**
         * This is the text displayed in the spinner element so make sure no debug information
         * is added.
         */
        @Override
        public String toString() {
            return mLanguageName;
        }
    }
}
